"use strict";

exports.__esModule = true;
exports.default = exports.transformReferences = exports.createImportDestructure = void 0;

var _template = _interopRequireDefault(require("@babel/template"));

var _utils = require("./utils");

var _core = require("@babel/core");

var _isReactHook = require("is-react-hook");

var _isHtmlTagName = _interopRequireDefault(require("is-html-tag-name"));

var _helperPluginUtils = require("@babel/helper-plugin-utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _extends() { _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; }; return _extends.apply(this, arguments); }

function _templateObject3() {
  var data = _taggedTemplateLiteralLoose(["\n  const {c: CNAME, s: SNAME, x: XNAME} = IMPORT_NAME\n"]);

  _templateObject3 = function _templateObject3() {
    return data;
  };

  return data;
}

function _templateObject2() {
  var data = _taggedTemplateLiteralLoose(["S(SUBJECT, STAGES, QUASI)"]);

  _templateObject2 = function _templateObject2() {
    return data;
  };

  return data;
}

function _templateObject() {
  var data = _taggedTemplateLiteralLoose(["S(SUBJECT, STAGES)"]);

  _templateObject = function _templateObject() {
    return data;
  };

  return data;
}

function _taggedTemplateLiteralLoose(strings, raw) { if (!raw) { raw = strings.slice(0); } strings.raw = raw; return strings; }

var PLUGIN_NAME = 'Sowing Machine';
var MODULE_NAME = 'sowing-machine';
var createCallStitch = (0, _template.default)(_templateObject());
var createTagTemplateStitch = (0, _template.default)(_templateObject2());
var createImportDestructure = (0, _template.default)(_templateObject3());
exports.createImportDestructure = createImportDestructure;
var typeToHelpersMap = {
  CallExpression: [_utils.fromCallExpression, createCallStitch],
  TaggedTemplateExpression: [_utils.fromTaggedTemplateExpression, createTagTemplateStitch]
};

var createVisitorFactory = function createVisitorFactory(s, x) {
  return function (referencePath) {
    return {
      'CallExpression|TaggedTemplateExpression': function CallExpressionTaggedTemplateExpression(path) {
        var node = path.node;

        if (_core.types.isCallExpression(node)) {
          var _name = node.callee.name;
          if (_name === s.name || (0, _isReactHook.isHookName)(_name)) return;
        }

        var _typeToHelpersMap$nod = typeToHelpersMap[node.type],
            assemble = _typeToHelpersMap$nod[0],
            create = _typeToHelpersMap$nod[1];

        var _assemble = assemble(node, x),
            callee = _assemble[0],
            STAGES = _assemble[1],
            QUASI = _assemble[2];

        var bindings = referencePath.scope.bindings;
        var name = callee.name;
        var isHTMLElement = !bindings[name] && (0, _isHtmlTagName.default)(name);
        var SUBJECT = isHTMLElement ? _core.types.stringLiteral(name) : callee;

        var templateArgs = _extends({
          S: s,
          SUBJECT: SUBJECT,
          STAGES: STAGES
        }, QUASI ? {
          QUASI: QUASI
        } : {});

        var replacement = create(templateArgs);
        path.replaceWith(replacement);
      }
    };
  };
};

var transformReferences = function transformReferences(referencePaths, state) {
  var c = state.c,
      s = state.s,
      x = state.x;
  var createVisitor = createVisitorFactory(s, x);

  for (var _iterator = referencePaths, _isArray = Array.isArray(_iterator), _i = 0, _iterator = _isArray ? _iterator : _iterator[Symbol.iterator]();;) {
    var _ref;

    if (_isArray) {
      if (_i >= _iterator.length) break;
      _ref = _iterator[_i++];
    } else {
      _i = _iterator.next();
      if (_i.done) break;
      _ref = _i.value;
    }

    var referencePath = _ref;
    var parentPath = referencePath.parentPath;
    var visitor = createVisitor(referencePath);
    parentPath.traverse(visitor, state);
    var args = parentPath.node.arguments;
    var first = args[0],
        rest = args.slice(1);
    var isContainer = _core.types.isArrowFunctionExpression(first) || _core.types.isFunctionExpression(first) || _core.types.isIdentifier(first) && rest.length === 0;
    var quasi = parentPath.parentPath.node.quasi;
    var replacement = isContainer ? [c, [_core.types.numericLiteral((0, _utils.tick)())].concat(args, quasi ? [quasi] : [])] : [s, args];
    (quasi ? parentPath.parentPath : parentPath).replaceWith(_core.types.callExpression.apply(_core.types, replacement));
  }
};

exports.transformReferences = transformReferences;

var _default = (0, _helperPluginUtils.declare)(function (api) {
  api.assertVersion(7);
  return {
    name: PLUGIN_NAME,
    visitor: {
      ImportDeclaration: function ImportDeclaration(path, state) {
        var node = path.node,
            scope = path.scope;

        if (node.source && node.source.value === MODULE_NAME) {
          var importName = node.specifiers[0].local.name;

          var _map = ['c', 's', 'x'].map(function (i) {
            return scope.generateUidIdentifier(i);
          }),
              c = _map[0],
              s = _map[1],
              x = _map[2];

          var defaultImport = path.container.shift();
          path.container.unshift(defaultImport, createImportDestructure({
            CNAME: c,
            SNAME: s,
            XNAME: x,
            IMPORT_NAME: importName
          }));
          var referencePaths = scope.bindings[importName].referencePaths;
          transformReferences(referencePaths, _extends({}, state, {
            c: c,
            s: s,
            x: x
          }));
        }
      }
    }
  };
});

exports.default = _default;